<?php
include "config.php";
# So, if you're reading this, there are a few things you should know:
#  1. I haven't used PHP in many years.
#  2. I haven't used SQL in many years.

# Some helpers for the printer
$FULL_CUT = "\x1Bi"; # Just cut wherever you are.  The cutter is after the print head, so you'll miss stuff.
$SAFE_FULL_CUT = "\x1DVA\x00\n"; # Feed the last printed stuff past the cutter, then cut.
function barcode($type, $message, $hri=true) {
	$rv = "";
	if ($hri) {
		$rv .= "\x1DH".chr(2);
	} else {
		$rv .= "\x1DH".chr(0);
	}
	$len = strlen($message);
	switch($type) {
		case "UPC-A":
			# UPC-A supports 11 or 12 digits
			if ($len < 11 || $len > 12) {
				throw new UnexpectedValueException("barcode() for UPC-A only supports message of length 11-12; '$message' is invalid.");
			}
			# TODO FIXME: Verify that the message is just a string of digits
			$rv .= "\x1DkA".chr($len)."${message}";
			break;
		case "CODE128":
			if ($len < 2 || $len > 255) {
				throw new UnexpectedValueException("barcode() for CODE128 only supports message of length 2-255; '$message' is invalid.");
			}
			# TODO FIXME The "{B" selects code set B, which can represent ASCII 20-7F (includes all
			# printable normal ASCII characters, but not the extended ones)
			# The right thing to do here is properly determine when to select which code set based
			# on the actual message (and switch code sets as needed).
			# TODO FIXME: validate the input message is printable
			$rv .= "\x1DkI".chr($len+2)."{B${message}";
			break;
		case "CODE39":
			if ($len < 1 || $len > 255) {
				throw new UnexpectedValueException("barcode() for CODE39 only supports message of length 2-255; '$message' is invalid.");
			}
			# CODE39 supports A-Z, 0-9, space, and -.$/+%
			# TODO FIXME validate message
			$rv .= "\x1DkE".chr($len)."${message}";
			#throw new UnexpectedValueException("barcode() doesn't support type '$type'");
			break;
		case "CODE93":
			if ($len < 1 || $len > 255) {
				throw new UnexpectedValueException("barcode() for CODE93 only supports message of length 2-255; '$message' is invalid.");
			}
			# CODE93 supports A-Z, 0-9, space, and -.$/+%
			# TODO FIXME validate message
			$rv .= "\x1DkH".chr($len)."${message}";
			break;
		default:
			throw new UnexpectedValueException("barcode() doesn't support type '$type'");
			break;
	}
	return $rv;
}


# Connect to the DB
$conn = new PDO(
	"mysql:host=$CONFIG_MYSQL_HOST;dbname=$CONFIG_MYSQL_DBNAME",
	"$CONFIG_MYSQL_USERNAME",
	"$CONFIG_MYSQL_PASSWORD",
	array(
	)
);
# Set the PDO error mode 
$conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

# Handle the request
if ($_POST["action"] == "classList") {
	$q = $conn->prepare("SELECT idclasses, classname, claytype, claytypes.description FROM classes LEFT JOIN claytypes ON classes.claytype = claytypes.idclaytypes WHERE active = 1 ORDER BY sortindex");
	$q->execute();
	$q->setFetchMode(PDO::FETCH_ASSOC);
	$rows = $q->fetchAll();
	$str = json_encode($rows);
	print $str;
} elseif ($_POST["action"] == "potterList") {
	$idclasses = $_POST["idclasses"];
	$q = $conn->prepare("SELECT idpotters, pottername FROM potters WHERE potterclass=:idclasses ORDER BY pottername");
	$q->bindValue(":idclasses", $idclasses, PDO::PARAM_INT);
	$q->execute();
	$q->setFetchMode(PDO::FETCH_ASSOC);
	$rows = $q->fetchAll();
	$str = json_encode($rows);
	print $str;
} elseif ($_POST["action"] == "clayList") {
	$q = $conn->prepare("SELECT idclays, claytype, claytypes.description, clayname FROM clays LEFT JOIN claytypes ON clays.claytype = claytypes.idclaytypes WHERE active = 1 ORDER BY clayname");
	$q->execute();
	$q->setFetchMode(PDO::FETCH_ASSOC);
	$rows = $q->fetchAll();
	$str = json_encode($rows);
	print $str;
} elseif ($_POST["action"] == "nameCheck") {
	$potterName = htmlspecialchars($_POST["potterName"]);
	$q = $conn->prepare("SELECT pottername, idclasses, classname FROM potters INNER JOIN classes ON potters.potterclass = classes.idclasses WHERE pottername=:name");
	$q->bindValue(":name", $potterName);
	$q->execute();
	$q->setFetchMode(PDO::FETCH_ASSOC);
	$rows = $q->fetchAll();
	$str = json_encode($rows);
	print $str;
} elseif ($_POST["action"] == "printSlip") {
	# First, handle the potter.  They might be new, or might have switched classes.
	$potterName = htmlspecialchars($_POST["potterName"]);
	$claimedClassId = $_POST["idclasses"];
	$q = $conn->prepare("SELECT idpotters, potterclass FROM potters WHERE pottername=:name LIMIT 1");
	$q->bindValue(":name", $potterName);
	$q->execute();
	$q->setFetchMode(PDO::FETCH_ASSOC);
	$rows = $q->fetchAll();
	if (count($rows) == 0) {
		# They're new.  Add them.
		$q = $conn->prepare("INSERT INTO potters (pottername, potterclass) VALUES (:potterName, :claimedClassID)");
		$q->bindValue(":potterName", $potterName);
		$q->bindValue(":claimedClassID", $claimedClassId);
		$q->execute();
		# lastInsertID returns the last insert ID ***from this database connection***, so there
		# shouldn't be any concurrency issues here.
		$knownPotterId = $conn->lastInsertID();
	} else {
		# They aren't new.  Update their class if necessary.
		$row = $rows[0];
		$knownClassId = $row["potterclass"];
		$knownPotterId = $row["idpotters"];
		if ($knownClassId != $claimedClassId) {
			# Update their class.
			$stmt = "UPDATE potters SET potterclass=:claimedClassId WHERE idpotters=:idpotters";
			$q = $conn->prepare($stmt);
			$q->bindValue(":claimedClassId", $claimedClassId);
			$q->bindValue(":idpotters", $knownPotterId);
			$q->execute();
		}
	}
	# This is dopey, but we need the class name for the slip... select it out of the DB
	$q = $conn->prepare("SELECT classname FROM classes WHERE idclasses=:idclasses");
	$q->bindValue(":idclasses", $claimedClassId);
	$q->execute();
	$q->setFetchMode(PDO::FETCH_ASSOC);
	$rows = $q->fetchAll();
	# TODO FIXME: verify we got a row back? and ONLY 1 row?
	$className = $rows[0]["classname"];

	# Next, insert the pot into the database
	$stmt = "INSERT INTO pots (potspotter, glazedate, potsclay, weight, description, glazes) 
                   VALUES (:potspotter, :glazedate, :potsclay, :weight, :description, :glazes)";
	$weight = $_POST["weight"];
	$description = htmlspecialchars($_POST["description"]);
	$glazes = htmlspecialchars($_POST["glazes"]);
	$q = $conn->prepare($stmt);
	$q->bindValue(":potspotter", $knownPotterId);
	# Don't bother trusting client dates.
	$glazeDate = date("Y-m-d H:i:s");
	$q->bindValue(":glazedate", $glazeDate);
	$q->bindValue(":potsclay", $_POST["idclays"]);
	$q->bindValue(":weight", $weight);
	$q->bindValue(":description", $description);
	$q->bindValue(":glazes", $glazes);
	$q->execute();
	$potId = $conn->lastInsertID();

	# Generate the printer commands
	#$printerCommand = "UPC-A:\n".barcode("UPC-A", "123456789011")."\n";
	#$printerCommand .= "UPC-A:\n".barcode("UPC-A", "123456789012")."\n";
	#$printerCommand .= "CODE 39:\n".barcode("CODE39", "HI CODE39.")."\n";
	#$printerCommand .= "CODE 93:\n".barcode("CODE93", "HI CODE93.")."\n";
	#$printerCommand .= "CODE 128:\n".barcode("CODE128", "Hello, World!")."\n";
	#$printerCommand .= "Cutting...\n\n\n\n$FULL_CUT";
	$nameWords = explode(" ", $potterName);
	$firstName = $nameWords[0];
	end($nameWords);
	$lastName = $nameWords[key($nameWords)];
	$barCodeStr = sprintf("%05d%'x3.3s%'x4.4s", $potId, $firstName, $lastName);
	# The CODE128 barcode can take about 13 chars, so do:
	# 1234567890123
	# [ID-]FIRLAST
	# 12345 supports printing IDs for 99999 pots in the database
	$wrappedDescription = wordwrap($description, 48);
	$wrappedGlazes = wordwrap($glazes,48);
	$printerCommand = "$potterName, $className\nDate: $glazeDate\nWeight: $weight\n$wrappedDescription\n$wrappedGlazes\n".barcode("CODE128", $barCodeStr).$SAFE_FULL_CUT;
	# Connect to the printer and print
	$connectTimeout = 5; # Only wait 5 seconds for the printer to respond.
	if (1 == 1) { # To disable printing for debug/development.
	$oldErrorReporting = error_reporting();
	# Suppress warnings, since we handle them
	error_reporting($oldErrorReporting ^ E_WARNING);
	$client = stream_socket_client("tcp://${CONFIG_PRINTER_IP}:$CONFIG_PRINTER_PORT", $errNo, $errMsg, $connectTimeout);
	if ($client == false) {
		$prettyDetails = "";
		if ($errNo == 10060) {
			$prettyDetails = "I couldn't connect to the printer (it didn't reply to me)<br>\n";
		} elseif ($errNo == 10061) {
			$prettyDetails = "I couldn't connect to the printer (it refused to talk to me)<br>\n";
		}
		$rv = array("errNo" => "1", "errMsg" => "Code: $errNo<br>\nMessage: $prettyDetails$errMsg", "msg" => "Sorry, something went wrong and you'll need to hand-write your glaze slip.  Please inform Kit/Judy/Andy.");
		$str = json_encode($rv);
		print $str;
		exit;
		#throw new UnexpectedValueException("Failed to connect to printer ($errNo): $errMsg");
	}
	error_reporting($oldErrorReporting);
	fwrite($client, $printerCommand);
	fclose($client);
	}
	# Pretend printing takes a few seconds
	sleep(1);
	$rv = array("errNo" => "0", "errMsg" => "Success!", "msg" => "Your slip is ready!");
	$str = json_encode($rv);
	print $str;
	
} elseif ($_POST["action"] == "adminPotterList") {
	$q = $conn->prepare("SELECT idpotters, pottername, potterclass FROM potters ORDER BY pottername");
	$q->execute();
	$q->setFetchMode(PDO::FETCH_ASSOC);
	$rows = $q->fetchAll();
	$str = json_encode($rows);
	print $str;
} elseif ($_POST["action"] == "adminUpdatePotter") {
	$idpotters = $_POST["idpotters"];
	$pottername = htmlspecialchars($_POST["pottername"]);
	$idclasses = $_POST["idclasses"];
	$potteractive = $_POST["idclasses"];
	$q = $conn->prepare("UPDATE potters SET pottername=:pottername, potterclass=:idclasses WHERE idpotters=:idpotters LIMIT 1");
	$q->bindValue(":idpotters", $idpotters, PDO::PARAM_INT);
	$q->bindValue(":pottername", $pottername);
	$q->bindValue(":idclasses", $idclasses, PDO::PARAM_INT);
	$q->execute();
	$rv = array("errNo" => "0", "errMsg" => "Success!", "msg" => "Data updated!");
	$str = json_encode($rv);
	print $str;
} elseif ($_POST["action"] == "adminAddPotter") {
	$pottername = htmlspecialchars($_POST["pottername"]);
	$idclasses = $_POST["idclasses"];
	$potteractive = $_POST["potteractive"];
	print "Potter name: $pottername\nidclasses: $idclasses\n";
	$q = $conn->prepare("INSERT INTO potters (pottername, potterclass) VALUES (:pottername, :idclasses)");
	$q->bindValue(":pottername", $pottername);
	$q->bindValue(":idclasses", $idclasses, PDO::PARAM_INT);
	$q->execute();
	$potterId = $conn->lastInsertID();
	$rv = array("errNo" => "0", "errMsg" => "Success!", "msg" => "Data updated!  New potter ID is " + $potterId);
	$str = json_encode($rv);
	print $str;
} else {
	print "On one hand, it looks like you asked me to do something I couldn't understand.  On the other hand, you have five fingers.";
}
?>
